"""A Python Library to Interface with the amaroK player as a script.

Author:  John Mulligan <phlogistonjohn@yahoo.com>
Licence: BSD


Goals
-----

The goal of this library is to wrap and simplify a lot of the 
annoying grunt work needed when writing an amaroK script in python
of moderate or high complexity. This includes wrapping DCOP
modules so that the best module installed on your system is automatically
chosen. This includes a fallback that should allow everyone with amaroK
to use the script without forcing them to install extra packages.

This also includes a class to create event loops based on amaroK's 
event strings written to the scripts standard input. Subclass this
and create handler methods for each event you want your script to be able
to deal with.


Sub-Modules
-----------

- mydcop: DCOP Abstration module. Use the functions here,
  they will automatically use the best dcop module on your system.
  
- scriptio: Event loop class for use in a script that listens for
  events from amaroK. Pretty usless for one-shot scripts though.

- pipedcop: DCOP via subprocess module. Don't use this directly, use
  mydcop.


amaroK Support
--------------

Currently this library should work with all versions of amaroK in
the 1.3.* series. If it doesn't please report a bug.


Requirements
------------

- amaroK: There insn't a strong reason to use this otherwise, is there?
- dcop (command line program): This should be installed as part of KDE
  but it is required for the fallback DCOP module
  

Optional Requirements
---------------------

- pydcop: Python module that directly interfaces with DCOP

"""



import mydcop

import scriptio

