
# Changed 2006-08-09 To work with amarok 1.4.2-beta1

DEF_MAX     = 12
DB_ENCODING = 'latin1'
BANNER_TXT  = 'Last.fm Suggests'
USE_GREET   = False


import amklib
import scrobxlib
import urllib
import itertools
import os,sys

from amklib.mydcop import ezSelect, dbQuote

popupMessage = amklib.mydcop.newMethod( 'amarok', 'playlist', 'popupMessage' )

#FMT = '<a href="%(url)s">%(name)s</a>'



class Limit (object):
    
    def __init__( self, count ):
        self.count = count
        self.curr  = 0

    def __call__( self, it ):
        if self.count is None:
            return it

        return self._gen( it )


    def _gen( self, it ):
        for item in it:
            print '(tick)'
            if self.curr >= self.count:
                return

            self.curr += 1
            yield item


        

class NotInCollection (Exception):
    pass



class Script (amklib.scriptio.ScriptLoop):
    
    greet = USE_GREET
    name = "Last.fm Suggests"
    heading = name + ':'
    count = DEF_MAX
    
    def setUp( self ):
        self.addMenuItem( "Any Similar Artist" )
        self.addMenuItem( "Similar Artists not in your Collection" )

        
        self.w = Worker()

        if self.greet:
            self.w.msg( "Hello", "Welcome to Last.fm Suggets :-)" )

            

    def tearDown( self ):
        self.clearMenuItem( "Any Similar Artist" )
        self.clearMenuItem( "Similar Artists not in your Collection" )

    def listing( self, iterable ):
        popupMessage( listing( self.heading, FMT, iterable, self.count ) )

    def handleMenu_Any_Similar_Artist( self, url ):
        self.w.listSimilarArtists( url )
    


    def handleMenu_Similar_Artists_not_in_your_Collection( self, url ):
        self.w.listSimilarArtistsNIC( url )
    
        

    def handle_configure( self, arg ):
        global DEF_MAX

        DEF_MAX = count_ui( DEF_MAX )
        print 'new count=', DEF_MAX




class Worker (object):

    cache = []
    db_type = ''
    die = False

    def __init__( self ):
        columns = ( 'id', 'type', 'label', 'lastmountpoint' )
        self.cache = list( amklib.mydcop.ezSelect( columns, 'devices' ) )

        # It turns out that amarok's devices system is funkier than I thought
        # we need to detect DB type (sqlite,mysql, etc) to find out
        # how to concatenate strings
        # -- Use a cheesy probe to detect it
        try:
            tst1 = amklib.mydcop.query( "SELECT 'testing' || '-123' " )[0]
        except:
            tst1 = ''
        try:
            tst2 = amklib.mydcop.query( "SELECT CONCAT('testing', '-123') " )[0]
        except:
            tst2 = ''

        if tst1 == 'testing-123':
            self.db_type = 'sqlite'
        elif tst2 == 'testing-123':
            self.db_type = 'mysql'
        else:
            self.db_type = 'unknown'

            

        
        if not self.cache:
            self.msg( "Error", "Last.fm Suggests 0.2 needs Amarok 1.4.2 & up" )
            self.die = True
            
   

    def msg( self, header, body ):
        popupMessage( '<b>%s</b><br>%s' % (header,body) )
    

    def oops( self, *txts ):
        print >>sys.stderr, 'OOPS:', ' '.join([str(ii) for ii in txts])
        
    

    def fileUrlTagIdent( self, fileUrl ):
        """deprecated?"""
        
        type, fileUrl = urllib.unquote_plus( fileUrl ).split( '://', 1 )

        if type != 'file':
            # can't handle things outside of the collection!
            return 0, ''
 

        dev_id, dev_mtpt = 0, ''
        
        for row in self.cache:
            if fileUrl.startswith( row[3] ) and row[3] > dev_mtpt:
                dev_id  = int(row[0])
                dev_mtpt = row[3]

        return dev_mtpt, fileUrl.replace( dev_mtpt, '.', 1 )


    def fileClause( self, fileUrl ):
        typ, path = urllib.unquote_plus( fileUrl ).split( '://', 1 )

        if typ != 'file':
            raise NotInCollection( "Url type invalid" )


        if self.db_type == 'sqlite':
            wh_clause = '( devices.lastmountpoint || "/" || substr(tags.url,3,999) )'
        else:
            wh_clause = 'CONCAT(devices.lastmountpoint,substr(tags.url,3,999))'

        return '%s LIKE %s' % ( wh_clause, dbQuote("%"+path) )


  


    def artistName( self, fileUrl ):
        
        get_cols   = ( 'artist.name', )
        get_tables = 'tags,artist,devices'

        where = """artist.id = tags.artist
                   AND tags.deviceid = devices.id
                   AND %s
                   """ % ( self.fileClause(fileUrl) )
        # is this really the best way to do it? ... oh well :-P


        results = ezSelect( get_cols,get_tables,where=where,limit=1,debug=1 )

        try:
            row = results.next()
        except StopIteration:
            raise NotInCollection( 'Url not in collection' )
        
        if not row[0]:
            raise ValueError( "Blank artist name" )
    
        return row[0]



    def hasArtist( self, artist ):
        where = 'name=%s' % amklib.mydcop.dbQuote( artist )
        results = ezSelect( ('name',), 'artist', where=where, debug=1 )    
        
        try:
            return  ( results.next()[0] == artist )
            
        except StopIteration:
            return False
    

       
    def genTxt( self, iterable ):

        out = ''
        for scrob in iterable:
            out += scrob.urlLink() + '<br>\n'

        if not out:
            return 'Sorry, no matches available<br>'


        return out
            
        
    


    # THE REAL STUFF

    def listSimilarArtists( self, url ):
        try:
            # get artist name
            artist = self.artistName( url )

            # set maximum number to return
            limit = Limit( DEF_MAX )

            # get last.fm results
            results = limit( ScrobSimilar(artist) )

            # generate message
            txt = str(self.genTxt(results))
            
            # display message
            self.msg( BANNER_TXT, txt )

        except NotInCollection, excp:
            self.oops( url, excp)
            self.msg( BANNER_TXT, "Sorry, that artist is unknown." )
        except ValueError, excp:
            self.oops( url, excp )
            self.msg( BANNER_TXT, "Error - Bad Artist Name (?)" )


    def listSimilarArtistsNIC( self, url ):
        
        def chk( scrob ):
            return self.hasArtist( scrob.artist )
    
        try:
            # get artist name
            artist = self.artistName( url )

            # set maximum number to return
            limit = Limit( DEF_MAX )

            # get last.fm results
            results = limit( ent for ent in ScrobSimilar(artist) 
                                 if not chk(ent) )

            # generate message
            txt = str(self.genTxt(results))
            
            # display message
            self.msg( BANNER_TXT, txt )


        except NotInCollection, excp:
            self.oops( url, excp )
            self.msg( BANNER_TXT, "Sorry, that artist is unknown." )
        except ValueError, excp:
            self.oops( url, excp )
            self.msg( BANNER_TXT, "Error - Bad Artist Name (?)" )





class ScrobEntry (object):

    def __init__( self, dct ):
        self.artist  = dct['name'].encode( DB_ENCODING )
        self.url     = dct['url'].encode( DB_ENCODING )


    def urlLink( self ):
        print '--> url link <--'
        return '<a href="%s">%s</a>' % (
            self.url,
            self.artist
            )
        


class ScrobSimilar (object):
    
    def __init__( self, artist ):
        self.artist = urllib.quote_plus( artist )

    def __iter__( self ):
        print 'Querying Last.fm...'

        for item in scrobxlib.similar( self.artist ):
            yield ScrobEntry( item )
        




def main():
    try:
        app = Script( "Last.fm Suggests" )
        app.start()
        return 0
    except Exception, excp:
        os.popen2( '''kdialog --error "Last.fm Suggests Fatal Error: 
        Please email phlogistonjohn@yahoo.com and report the crash."'''  )
        raise excp
        return 1




def count_ui( default='' ):
    """NOT IMPL."""
    a,b = os.popen2( 'kdialog --inputbox "Number of suggestions:" %s' % default )
    for line in b:
        return int( line ) 



        

if __name__ == "__main__":
    sys.exit( main() )


